<?php
/**
 * User handler
 *
 */
class VSCAN_Company
{
	const ALL_COMPANIES_ID = 0;
	
	public static function getAllCompaniesName()
	{
		$companies = new VSCAN_Model_Company();
		$companies = $companies->fetchAll();
		$companiesNames = array();
		foreach ($companies as $company) {
	    	$companiesNames[$company->getId()] = $company->getName();
	    }
	    return $companiesNames;	
	}
	
	/**
	 * 
	 * return all campanies depending of the role user
	 * @param VSCAN_Model_User $user
	 * @return array of companies name
	 */
	public static function getAllCompaniesNameByUser(VSCAN_Model_User $user) {
		$acl = new VSCAN_Acl($user);
		if ($acl->isUserAllowed('allCompanies')) {
			$companies = VSCAN_Company::getAllCompaniesName();
		} else {
			$companies = VSCAN_User::getCompaniesNamesByUserId($user->id);
		}
		return $companies;
	}
	/**
	 * 
	 * delete the company if there are no normal users related to it
	 * @param int $companyId
	 */
	public static function delete($companyId)
	{
		$db = Zend_Db_Table::getDefaultAdapter();
		$db->beginTransaction();
		try {
			
			$companyUsers = new VSCAN_Model_CompanyUser();
			$companyUsers = $companyUsers->fetchAll(
				$companyUsers->select()->where(' company_id = ? ', $companyId)
			);
			$usersDeleteCompany = array();
			foreach($companyUsers as $companyUser) {
				$acl = new VSCAN_Acl($companyUser->getUser());
				if (!$acl->hasRole(VSCAN_Acl::ADMIN_ROLE)) {
					//normal users
					throw new Zend_Exception('This company has users related, so you can not delete it');
				}else{
					//these users should be admin
					$usersDeleteCompany[] = $companyUser->getUser()->getId();
				}
			}
			
			//remove company from admin users
			self::deleteCompanyForUsers($usersDeleteCompany, $companyId);
			
			self::deleteTargetsByCompany($companyId);
			$company = new VSCAN_Model_Company();
			$company->find($companyId);
			$company->delete();
			$db->commit();
		} catch (Zend_Exception $e) {
			$db->rollBack();
			throw $e;
		}
	}
	
	/**
	 * 
	 * get the scans value for a given company
	 * @param int $companyId
	 */
	public static function getAvailableScans($companyId) {
        $company = new VSCAN_Model_Company();
        $company->find($companyId);
        return $company->getScans();
	}
	
	public static function decrementScans($companyId){
		$db = Zend_Db_Table::getDefaultAdapter();
		$db->beginTransaction();
		try{
			$company = new VSCAN_Model_Company();
	        $company->find($companyId);
	        $company->setScans($company->getScans() - 1);
	        $company->save();
	        $db->commit();
		} catch (Zend_Exception $e) {
			$db->rollBack();
			throw $e;
		}
	}
	
	public static function incrementScans($companyId){
		$company = new VSCAN_Model_Company();
        $company->find($companyId);
        $company->setScans($company->getScans() + 1);
        $company->save();
	}

	public static function redirectToHomeUnlessResourceBelongToCompany($company_id){
		if (VSCAN_Sessions::getCurrentCompany() !=  $company_id){
             Zend_Controller_Action_HelperBroker::getStaticHelper('redirector')
             	->setCode(303)
             	->setExit(true)
      	        ->setGotoSimple("index", "index");
      	        return true;      	        
        }
        return false;
	}
	/**
	 * 
	 * Remove company from admin users
	 * @param array $usersDeleteCompany id of users
	 * @param int $companyId
	 */
	private static function deleteCompanyForUsers($usersDeleteCompany, $companyId)
	{
		$companyUsers = new VSCAN_Model_CompanyUser();
			foreach ($usersDeleteCompany as $userId) {
				
				$companyUsers->fetchRow(
					$companyUsers->select()
						->where(' company_id = ? ', $companyId)
						->where(' user_id = ? ', $userId)
				)->delete();
			}
	}
	/**
	 * 
	 * cascade delete
	 * @param int $companyId
	 */
	private static function deleteTargetsByCompany($companyId)
	{
		$targets = new VSCAN_Model_Target();
		$targets = $targets->fetchAll(
			$targets->select()->where('company_id = ?', $companyId)
		);
		
		foreach ($targets as $target) {
			$target->delete();
		}
	}
	
}