<?php
/**
 * 
 * handle nessus comiunication
 */
abstract class VSCAN_Nessus_Abstract {
	
	const REPORT_COMPLETED = 'completed';
	/**
	 * 
	 * Http client to make the calls
	 * @var Zend_Http_Client
	 */
	protected $_curl;
	protected $_curlOptions;
	protected $_url;
	protected $_user;
	protected $_password;
	
	protected $_auth_token;
	
	public function __construct($user = null, $password = null, $url = null) {
		if (empty($url)) {
			$url = Zend_Registry::get('config')->nessus->url;
		}
		if (empty($user)) {
			$user = Zend_Registry::get('config')->nessus->user;
		}
		if (empty($password)) {
			$password = Zend_Registry::get('config')->nessus->password;
		}
	
		$this->_url = $url;
		$this->_user = $user;
		$this->_password = $password;
		$this->initCurl();
	}
	
	public function setCurlOptions(array $options) {
		$this->_curlOptions = $options;
	}
	
	
	public function downloadReport($uuid)
	{
	    $this->prepareCurlForNessus();
		
		if (!$this->isReportCompleted($uuid)) return null;
		$this->_curl->setUri($this->_url . "/file/report/download");
		$post_data = array("report" => $uuid,  "v2" => "v2");
		$this->_curl->setParameterPost($post_data);
		$response = $this->_curl->request('POST')->getBody();
		$xmlresponse = simplexml_load_string($response);
		
		if(!$xmlresponse) return null;
        return $xmlresponse;
	}
	/**
	 * 
	 *	return true if the scan is completed and the report is ready to use
	 * @param string $uuid uuid scan
	 */
	protected function isReportCompleted($uuid)
	{
		$this->prepareCurlForNessus();
	    $this->_curl->setUri($this->_url . "/report/list");
	    $post_data = array("seq" => uniqid());
	    $this->_curl->setParameterPost($post_data);
	    $response = $this->_curl->request('POST')->getBody();
	    $list = simplexml_load_string($response);
	    $report = $list->xpath("//*[text()='$uuid']/..");
	    if (empty($report)) return false;
	    return ($report[0]->status == self::REPORT_COMPLETED);
	} 
	/**
	 * 
	 * create token
	 *
	 */
	protected function createNessusSession()
	{
		$this->_curl->getAdapter()->setConfig($this->_getCurlOptions());
		$this->_curl->setUri($this->_url . "/login");
		$post_data = array("login" => $this->_user, "password" => $this->_password, "seq" => uniqid());
		$this->_curl->setParameterPost($post_data);
		$response = $this->_curl->request('POST');
		$xmlresponse = simplexml_load_string($response->getBody());
		$this->_auth_token = $xmlresponse->contents->token;
		return  $this->_auth_token;
	}
	
	protected function initCurl() {
		$adapter = new Zend_Http_Client_Adapter_Curl();
		$this->_curl = new Zend_Http_Client();
		//$this->_curl->setHeaders('Content-Type', $contentType);
		$this->_curl->setAdapter($adapter);
		//$this->_curl->getAdapter()->setConfig($options);
	}
	
	/**
	 * 
	 * prepare default curl call for nessus server
	 */
	protected function prepareCurlForNessus(){
		if (empty($this->_auth_token)) {
	    	$this->CreateNessusSession();
	    }
		$curlConfig = $this->_getCurlOptions();
	    $curlConfig['curloptions'][CURLOPT_COOKIE] = "token=$this->_auth_token";
		$this->_curl->getAdapter()->setConfig($curlConfig);
	}
	protected function _getCurlOptions() {
		
		return array (
			'curloptions' => array(
				CURLOPT_SSL_VERIFYHOST => 1,
				CURLOPT_SSL_VERIFYPEER => false,
				CURLOPT_RETURNTRANSFER => true,
				CURLOPT_CONNECTTIMEOUT => 30
			)
		);
	}
}