<?php

class VSCAN_Report {
	
	const RISK_NONE = 'none';
	const RISK_LOW = 'low';
	const RISK_MEDIUM = 'medium';
	const RISK_HIGH = 'high';
	const RISK_CRITICAL = 'critical';

	public static $HIDDEN_FIELDS_DATA = array(
		//do not remove these lines
		'fname',
		'plugin_modification_date',
		'plugin_name',
		'plugin_publication_date',
		'plugin_type'
	);
	/**
	 * 
	 * @use CRONJOB
	 * get report of scan complete
	 * @return array Scans
	 */
	public static function retrieveReports(){
		$scans = VSCAN_Scan::getRunningScans();
		$reports = array();
		$ws = new VSCAN_WS_Reports();
		foreach ($scans as $scan) {
			try {
				$uuid = $scan->getUuid();
				$xmlReport = $ws->getReportXML($uuid);
				if (empty($xmlReport)) {
					$reports[$uuid] = VSCAN_Scan::RUNNING;
					continue;
				}
				$reports[$uuid] = self::buildReportByXML($xmlReport, $scan->getId());
				$scan->setStatus(VSCAN_Scan::FINISHED);
				$scan->save();
				self::sendMail($scan);
			} catch (Exception $e) {
				//TODO:show be logged and continue with the rest of the scans
				echo $e->getMessage();
			}
			
		}
		return $reports;
	}
	
	/**
	 * 
	 * @use CRONJOB
	 * get reports of customScan complete
	 * @return array Scans
	 */
	public static function retrieveCustomReports(){
		$reportItems = VSCAN_Scan::getRunningCustomScans();
		//$scans = VSCAN_Scan::getRunningScans();
		$reports = array();
		$ws = new VSCAN_WS_Reports();
		foreach ($reportItems as $reportItem) {
			try {
				$uuid = $reportItem->getUuid();
				$xmlReport = $ws->getReportXML($uuid);
				if (empty($xmlReport)) {
					$reports[$uuid] = VSCAN_Scan::RUNNING;
					continue;
				}
				$reports[$uuid] = (int)self::isCustomReportSolved($xmlReport, $reportItem);
			} catch (Exception $e) {
				//TODO:show be logged and continue with the rest of the scans
				echo $e->getMessage();
			}
			
		}
		print_r($reports);
		return $reports;
	}
	
	public static function getLastScanIdFinished($companyId){
		$scan = new VSCAN_Model_Scan();
		$select = $scan->select()->where('company_id = ?', $companyId)->where('status = ?', VSCAN_Scan::FINISHED)->order('created DESC');
		$scan->fetchRow($select);
		if (!$scan->isMapped()) return null;
		return $scan->getId();
	}
	public static function sendMail($scan){
		//for admin
		$values = array(
			'date'=>$scan->getDate(),
			'user_fullname' => $scan->getUser()->getFullname()
		);
		$mail = new VSCAN_Mail($values, 'The scan has finished', null, VSCAN_Mail::SCAN_FINISHED_TEMPLATE);
		$mail->send();
		//for users
		$company_id = $scan->getCompany()->getId();
		$company_users = new VSCAN_Model_CompanyUser();
		$company_users = $company_users->fetchAll($company_users->select()->where('company_id = ?', $company_id));
		
		$value_for_mail = array(
			'vscan_url_home' => Zend_Registry::get('config')->host->name,
			'vscan_url_report' => Zend_Registry::get('config')->host->name
		);

		foreach ($company_users as $company_user) {
			if ($company_user->getUser()->getActive()){
				$value_for_mail['user_fullname'] = $company_user->getUser()->getFullname();
				$mail = new VSCAN_Mail(
					$value_for_mail, 
					'VScan - New Vulnerability Assessment has been published', 
					$company_user->getUser()->getEmail(), 
					VSCAN_Mail::SCAN_FINISHED_TEMPLATE_FOR_USERS
				);
				
				$mail->send();	
			}
			
		}


	}
	/**
	 * 
	 * get last reports for a given company (the last Scan finished)
	 * @param unknown_type $companyId
	 */
	public static function getLastReport($companyId, &$scanId) {
		
		if ($scanId === null){
			$scanId = self::getLastScanIdFinished($companyId);
		}
		if ($scanId === null) return null;
	
		$report = new VSCAN_Model_Report();
		$subSelect = $report->select()->setIntegrityCheck(false)
			->from(array('r'=>'reports'), 
				array( 
					'ID' => 'r.id',
					'SUM(IF (report_items.risk = 4, 5, 0)) as risk4',
					'SUM(IF (report_items.risk = 3, 4, 0)) as risk3',
					'SUM(IF (report_items.risk = 2, 3, 0)) as risk2',
					'SUM(IF (report_items.risk = 1, 2, 0)) as risk1',
					'SUM(IF (report_items.risk = 0, 1, 0)) as risk0',
				

				)
			)
			->join(array('report_items'), 'report_items.report_id = r.ID and report_items.status <> "solved" ', array())
			->where('scan_id = ?', $scanId)
			->group('r.ID')
			->order('risk4 desc')->order('risk3 desc')->order('risk2 desc')->order('risk1 desc')->order('risk0 desc');
		$select = $report->select()->setIntegrityCheck(false)
		->distinct()->from('reports')
		->join(array('rs' => $subSelect), 'rs.ID = reports.id', array());
		
		
		$reports = $report->fetchAllForPage($select);
		
		return $reports;			
	}
	/**
	 * 
	 * 
	 * @param int $scan_id
	 * @return array reports
	 */
	public static function getReportsByScan($scan_id) {
		return self::getLastReport(null, $scan_id);
		$report = new VSCAN_Model_Report();
		$reports = $report->fetchAllForPage($report->select()->where('scan_id = ?', $scan_id)->order('created DESC'));
		return $reports;
	}
	
	/**
	 * 
	 * getAllfinish Scans for pagination
	 * @param unknown_type $companyId
	 */
	public static function getAllReports($companyId) {
	
			$scan = new VSCAN_Model_Scan();
			$select = $scan->select()->where('company_id = ?', $companyId)->where('status = ?', VSCAN_Scan::FINISHED)->order('created DESC');
			$scans = $scan->fetchAllForPage($select);
			return $scans;
			
	}
	/**
	 * 
	 * get reportItems for a given reportID and CompanyId
	 * @param int $reportId
	 * @param int $companyId
	 * @return array of reportItems
	 */
	public static function getDetailsReport($reportId, $companyId, &$reportName) {
			
			$report = new VSCAN_Model_Report();
			$select = $report->select()->where('id = ?', $reportId);	
			$report->fetchRow($select);
			if (!$report->isMapped()) return null;
			if ($companyId != $report->getScan()->getCompany()->getId()) return null;
			$reportName = $report->getName();
			$reportItem = new VSCAN_Model_ReportItem();
			if (!VSCAN_Sessions::getShowSolved()){
				$select = $reportItem->select()->where('report_id = ?', $report->getId())->where('status <> ? ', VSCAN_Scan::SOLVED)->order('risk DESC')->order('port')->order('description');
				$reportItems = $reportItem->fetchAllForPage($select);	
			} else {
				$reportItems = $reportItem->fetchAllForPage($reportItem->select()->where('report_id = ?', $report->getId())->order('risk DESC')->order('port')->order('description'));	
			}

			
			return $reportItems;
			
	}
	
	/**
	 * 
	 * create a report for a given xml
	 * @param SimpleXMLElement $xml
	 * @param int $scanId
	 * @throws Exception
	 * @return VSCAN_Model_Report
	 */
	protected static function buildReportByXML($xml, $scanId) {
		
		$db = Zend_Db_Table::getDefaultAdapter();
		$db->beginTransaction();
		try {
			//check errors in the xml
			if (empty($xml->Report->ReportHost[0])) {
				throw new Exception("XML incompleto ReportHost doesn't exist");
			}
			
			foreach ($xml->Report->ReportHost as $reportHost) {
				$name = (string)$reportHost['name'];
				$report = new VSCAN_Model_Report();
				$report->setName($name);
				$report->setScan($scanId);
				$report->save();
				
				foreach ($reportHost->ReportItem as $reportItem) {

					self::saveReportItem($reportItem, $report);
				}
			}
			$db->commit();
			return $report;
		} catch (Exception $e) {
			echo $e->getMessage();
			$db->rollBack();
			$scan = new VSCAN_Model_Scan();
			$scan->find($scanId);
			$scan->setStatus(VSCAN_Scan::FAILED);
			$scan->save();
			throw $e;
		}
		
		
	}
	/**
	 * 
	 * Is the reportintem is not resolved, it will be updated
	 * @param simple_XML $xml
	 * @param VSCAN_Model_ReportItem $reportItem
	 * @return boolean
	 */
	protected function isCustomReportSolved($xml, VSCAN_Model_ReportItem $reportItem){
		//check errors in the xml
		if (empty($xml->Report)) {	
			throw new Exception("XML incompleto ReportHost doesn't exist");
		}

		$attr = Zend_Json_Decoder::decode($reportItem->getAttributes());
		$md5ReportItemOriginal = md5($attr['port'] . $attr['pluginID']); 

		if (isset($xml->Report->ReportHost->ReportItem)) {

			foreach($xml->Report->ReportHost->ReportItem as $reportItem1){
				
				$data = self::simplexml2array($reportItem1);
				$md5ReportItem = md5($data['@attributes']['port'] . $data['@attributes']['pluginID']);
				
				if ($md5ReportItemOriginal != $md5ReportItem) {
					continue;
				}
				self::saveReportItem($reportItem1, null, $reportItem);
				/*$attributes = Zend_Json_Encoder::encode($data['@attributes']);
				
				unset($data['@attributes']);
				$data = Zend_Json_Encoder::encode($data);
				$reportItem->setAttributes($attributes);
				$reportItem->setData($data);
				$reportItem->setStatus(VSCAN_Scan::UNSOLVED);
				$reportItem->save();*/	
				return false;

			}
		}
		$reportItem->setStatus(VSCAN_Scan::SOLVED);
		$reportItem->save();
		return true;
	}
	/**
	 * 
	 * convert xml to a simple array
	 * @param SimpleXMLElement $xml
	 * @return array;
	 */
	private static function simplexml2array($xml) {
		if (is_object($xml)) {
			if (get_class($xml) == 'SimpleXMLElement') {
				$attributes = $xml->attributes();
				foreach($attributes as $k=>$v) {
					if ($v) $a[$k] = (string) $v;
				}
				$x = $xml;
				$xml = get_object_vars($xml);
			}
		}
		if (is_array($xml)) {
			if (count($xml) == 0) return (string) $x; // for CDATA
			foreach($xml as $key=>$value) {
				$r[$key] = self::simplexml2array($value);
			}
			if (isset($a)) $r['@attributes'] = $a;    // Attributes
			return $r;
		}
		return trim($xml);
	}
	
	/**
	 * 
	 * Data nessus interpreter
	 * Hard to define a correct parser, here we include some of our euristic
	 * @param unknown_type $data
	 */
	private static function parseDataToXml($data) {
		$data = str_replace('\n', "\n", $data);
		$pattern_key = '/^[A-Z][a-z\s]{1,25}\s:$\n/m'; //keys
		$pattern_clen_key = '/[^A-Za-z0-9\s]|\n/';
		preg_match_all($pattern_key, $data, $keys);
		$values = preg_split($pattern_key, $data);
		$data_ar = array();
		foreach ($keys[0] as $index=>$key) {
			$data_ar[trim(preg_replace($pattern_clen_key, '', $key))] = trim($values[$index+1]);
		}
		return $data_ar; 
	}
	/**
	 * convert risk to int to get a useful order in the table presentation
	 * @param string $stringRisk
	 */
	private static function riskToInt($stringRisk){

		switch (strtolower($stringRisk)){
			case self::RISK_NONE:
				return 0;
			case self::RISK_LOW:
				return 1;
			case self::RISK_MEDIUM:
				return 2;
			case self::RISK_HIGH:
				return 3;
			case self::RISK_CRITICAL:
				return 4;
			default:
				throw new Exception("$stringRisk is a unknown type of risk");
		}
	}
	/**
	 * @param $xmlReportItem
	 * @param $report
	 */
	private static function saveReportItem($xmlReportItem, $report = null, $reportItemOri = null){
		//TODO: refactor ReportItemPaser
		$data = self::simplexml2array($xmlReportItem[0]);
		
		
		$modelReportItem = (empty($reportItemOri))? new VSCAN_Model_ReportItem(): $reportItemOri;
		//Fields for order, useful to listview
		
		$modelReportItem->setPort($data['@attributes']['port']);
		$modelReportItem->setRisk(self::riskToInt($data['risk_factor']));
		$modelReportItem->setDescription($data['description']);
		
		
		$attributes = Zend_Json_Encoder::encode(self::replacing($data['@attributes']));
		unset($data['@attributes']);
		$data = Zend_Json_Encoder::encode(self::sortData(self::replacing($data)));
		

		
		$modelReportItem->setAttributes($attributes);
		$modelReportItem->setData($data);
		if (self::isReportItemMarkedAsOmittedPreviously($modelReportItem, $report)){
			$modelReportItem->setStatus(VSCAN_Scan::OMITTED);
			$modelReportItem->setReport($report);
		}else if (!empty($report)) {
			$modelReportItem->setStatus(VSCAN_Scan::UNRECHECKED);
			$modelReportItem->setReport($report);
		}else{
			$modelReportItem->setStatus(VSCAN_Scan::UNSOLVED);
		}

		$modelReportItem->save();
	}

	private static function isReportItemMarkedAsOmittedPreviously($modelReportItem, $report){
		try {

			$attr = Zend_Json_Decoder::decode($modelReportItem->getAttributes());
			$current_pluginID = $attr['pluginID'];
			$current_port = $modelReportItem->getPort();
			$current_host = $report->getName();
			$companyId = $report->getScan()->getCompany()->getId();
			$scanId = self::getLastScanIdFinished($companyId);
			
			if ($scanId == null) return false;

			$report = new VSCAN_Model_Report();
			$report = $report->fetchRow($report->select()->where('scan_id = ?', $scanId)->where('name = ?', $current_host));
			
			if (!$report->isMapped()) return false;

			$reportId = $report->getId();
			$reportItem = new VSCAN_Model_ReportItem();
			$select = $reportItem->select()->where('report_id = ?',$reportId)->where('port = ?', $current_port )->where('status = ?', VSCAN_Scan::OMITTED)->where('attributes like ?','%"pluginID":"'.$current_pluginID.'"%');
			$reportItem = $reportItem->fetchRow($select);
			
		} catch(Exception $e){

			echo $e->getMessage();
			die();
		}
		
		return $reportItem->isMapped();
	}
	/**
	*order the data by OrderArray (inside method)
	*/
	private static function sortData($data) {
    	$orderArray = array(
	    	'synopsis',
	    	'description', 
	    	'plugin_output',
	    	'cvss_base_score',
	    	'cvss_vector',
	    	'osvdb',
	    	'xref',
	    	'vuln_publication_date',
	    	'solution',
	    	'see_also'
	    );
    	
    	$ordered = array();
	    foreach($orderArray as $key) {
	    	if(array_key_exists($key, $data)) {
	    		$ordered[$key] = $data[$key];
	    		unset($data[$key]);
	    	}

	    }
	    return $ordered + $data;
	}
	/**
	*
	*/
	private static function replacing(&$arrayToClean){
		return $arrayToClean;
	}


}
