<?php

class VSCAN_Scan {
	/*
	 * different status
	 */
	const READY = 'ready';
	const STOPPED = 'stopped';
	const RUNNING = 'running';
	const FINISHED = 'finished';
	const FAILED = 'failed';
	const SOLVED = 'solved';
	const UNSOLVED = 'unsolved';
	const UNRECHECKED = 'unrechecked';
	const READY_FOR_SCAN = 'ready_for_scan';
	const OMITTED = 'omitted';
	
	/**
	 * 
	 * nessus key to find the report of an scan
	 * 
	 */
	const UUID = 'uuid';
	
	
	const FULLSCAN = 'FullScan';
	const CUSTOMSCAN = 'CustomScan';
	/**
	 * 
	 * delete scan and increment the company->scan value
	 * @param int $scanId
	 * @throws Zend_Exception
	 */
	public static function delete($scanId){
		$db = Zend_Db_Table::getDefaultAdapter();
		$db->beginTransaction();
		try {
			$scan = new VSCAN_Model_Scan();
			$scan->find($scanId);
			if ($scan->getStatus() == self::READY || $scan->getStatus() == self::STOPPED) {
				VSCAN_Company::incrementScans($scan->getCompany()->getId());
				$scan->delete();
			}else{
				throw new Zend_Exception('The status must be Ready or Stopped to can delete it');
			}
			$db->commit();
		}catch(Exception $e) {
			$db->rollBack();
			throw $e;
		}
	}
	/**
	 * 
	 * create scan decrementing scan company
	 * @param array $params scans attr
	 * @throws Zend_Exception
	 * @throws Exception
	 */
	public static function create($params) {
		$db = Zend_Db_Table::getDefaultAdapter();
		$db->beginTransaction();
		try {
			//force the status for a new scan
			$params['status'] = self::READY; 
			$scan = new VSCAN_Model_Scan($params);
			
			$company =  new VSCAN_Model_Company();
			$company->find($params['company']);
			if ($company->getScans() <= 0) {
				throw new Zend_Exception('There are no available scans');
			}
			$company->setScans($company->getScans() - 1);
			$company->save();
			$scan->save();
			$db->commit();
		}catch(Exception $e) {
			$db->rollBack();
			throw $e;
		}
		
	}
	
	public static function edit($params) {
		$scan = new VSCAN_Model_Scan();
		$scan->find($params['id']);
		if ($scan->getStatus() == self::READY || $scan->getStatus() == self::STOPPED || $scan->getStatus() == self::FAILED) {
			$scan = new VSCAN_Model_Scan();
			$params['status'] = self::READY;
        	$scan->populate($params);
        	$scan->save();
		} else {
			throw new Zend_Exception('The status of the Scan must be Ready or Stopped');
		}
		
	}
	/**
	 * 
	 * stop a running scan
	 * @param int $scanId
	 * @throws Zend_Exception
	 */
	public static function stop($scanId) {
		
		$scan = new VSCAN_Model_Scan();
		$scan->find($scanId);
		if ($scan->getStatus() != self::RUNNING) {
			throw new Zend_Exception('This scan is not running');
		}
		$scan->setStatus(self::STOPPED);
		
		$ws = new VSCAN_WS_Scans();
		
		$ws->Stop($scan->getUuid(), $scan->getCompany()->getName());
		$scan->save();
		
	} 
	
	/**
	 * 
	 * @use CRONJOB
	 * Execute the scans availables
	 * @return array Scans
	 */
	public static function runScans(){
		$scans = self::getReadyScansToRun();
		$run = array();
		$ws = new VSCAN_WS_Scans();
		foreach ($scans as $scan) {
			$companyId = $scan->getCompany()->getId();
			$target = new VSCAN_Model_Target();
			$targets = $target->fetchAll($target->select()->where('company_id = ? ',$companyId));
			$aTargets = array();
			$targetListName = array();
			foreach ($targets as $target) {
				$target = $target->toArray();
				unset ($target['company']);
				unset ($target['id']);
				$tagetName = $target['start'];
				if (!empty($target['end'])){
					$tagetName .= ' - ' . $target['end'];
				}
				$targetListName[] = $tagetName;
				//default variables TODO: refactoring
				$target['pluginid'] = null; //need for custom 
				$target['port'] = null; //need for custom
				$target['reporthostcod'] = null;
				$target['reporthostitem'] = null;
				$target['loginpage'] = null;
				$target['loginform'] = null;
				$target['loginformfields'] = null;
				$target['loginformmethod'] = null;
				$target['checkauthonpage'] = null;
				$aTargets[] = $target;
			}

			$scan->setTargets(implode(',', $targetListName));
			print_r($aTargets);
			$aScan = $scan->toArray();
			unset($aScan['user']);
			$aScan['company'] = $aScan['company']->getName();
			$run[] = $aScan;
			$scan->setStatus(self::RUNNING);
			$scan->save();
			$ret = $ws->VScan($aScan['id'], $aScan['company'], self::FULLSCAN , $aTargets);
			if ($ret) {
				$scan->setUuid($ret);
			}else {
				$scan->setStatus(self::FAILED);
				//TODO: Implement Log
				echo 'ERROR: ', $scan->getId(), print_r($ret, true);
			}
			$scan->save();
			
		}
		return $run;
	}

	public static function markForCustomScan($reportItemId, $companyId){
		$reportItem = new VSCAN_Model_ReportItem();
		$reportItem->find($reportItemId);
		if(!$reportItem->isMapped()) throw new  Exception('Vulnerability not found');
		//checking if the vul belong to the last report of their company
		$reportId = $reportItem->getReport()->getId();
		$reports = new VSCAN_Model_Report();
		$scan = new VSCAN_Model_Scan();
		$scan = $scan->fetchRow($scan->select()->where('company_id = ?', $companyId)->where('status = ?', VSCAN_Scan::FINISHED)->order('id DESC'));
		if ($scan->getId() != $reportItem->getReport()->getScan()->getId()) {
			throw new Exception('The functionality of rechecking vulnerabilities is only available for the last report of an active group.');
		}
		$company = $reportItem->getReport()->getScan()->getCompany();

		if ($company->getId() != $companyId) {
			throw new Exception("The vulnerability is out of the scope of the group");
		}
		
		try{
			$reportItem->setStatus(VSCAN_Scan::READY_FOR_SCAN);
			$reportItem->save();
		}catch(Exception $e){
			return 0;
		}
		return 1;
	}

	public static function markVulAsOmitted($reportItemId, $companyId){
		$reportItem = new VSCAN_Model_ReportItem();
		$reportItem->find($reportItemId);

		$reportId = $reportItem->getReport()->getId();
		$reports = new VSCAN_Model_Report();
		$scan = new VSCAN_Model_Scan();
		$scan = $scan->fetchRow($scan->select()->where('company_id = ?', $companyId)->where('status = ?', VSCAN_Scan::FINISHED)->order('id DESC'));
		if ($scan->getId() != $reportItem->getReport()->getScan()->getId()) {
			throw new Exception('The functionality of omitting vulnerabilities is only available for the last report of an active group.');
		}
		$company = $reportItem->getReport()->getScan()->getCompany();

		if ($company->getId() != $companyId) {
			throw new Exception("The vulnerability is out of the scope of the group");
		}

		try{
			$reportItem->setStatus(VSCAN_Scan::OMITTED);
			$reportItem->save();
		}catch(Exception $e){
			return 0;
		}
		return 1;

	}

	public static function runCustomScans(){
		$reportItem = new VSCAN_Model_ReportItem();
		$reportItems = $reportItem->fetchAll($reportItem->select()->where('status = ?', VSCAN_Scan::READY_FOR_SCAN)->order('modified'));
		foreach ($reportItems as $reportItem){
			$reportItem->setStatus(self::RUNNING);
			$reportItem->save();	
		}
		/** Workaround FGM to avoid multiples recheck of the same item **/
		foreach ($reportItems as $reportItem){
			self::runCustomScan($reportItem);
		}
	}
	/**
	 * 
	 * 
	 * Execute a customscan vulnerability
	 * @return array Scans
	 */
	public static function runCustomScan($reportItem) {
		
		$company = $reportItem->getReport()->getScan()->getCompany();

		$attr = json_decode($reportItem->getAttributes());
		$target = array(
			'type' => VSCAN_Target::HOSTNAME_KEY,
			'start' => $reportItem->getReport()->getName(),
			'end' => null,
			'port' => $attr->port,
			'pluginid' => $attr->pluginID
		);
		
		$ws = new VSCAN_WS_Scans();
		$ret = $ws->VScan($reportItem->getId(), $company->getName(), self::CUSTOMSCAN , array($target));
		if ($ret) {
			$reportItem->setUuid($ret);
		} else {
			$reportItem->setStatus(self::FAILED);
		}
		$reportItem->save();
		return $ret;
	} 
	/**
	 * 
	 * return running scans, without a report
	 */
	public static function getRunningScans() {
		$scans = new VSCAN_Model_Scan();
		$scans = $scans->fetchAll($scans->select()->where('status = ?', self::RUNNING));
		return $scans;
	}
	
	/**
	 * 
	 * return running scans, without a report
	 */
	public static function getRunningCustomScans() {
		$reportItems = new VSCAN_Model_ReportItem();
		$reportItems = $reportItems->fetchAll($reportItems->select()->where('status = ?', self::RUNNING));
		return $reportItems;
	}
	
	/**
	 * 
	 * return the scan in the state ready scheduled for the current date time
	 */
	private static function getReadyScansToRun(){
	
		$scans = new VSCAN_Model_Scan();
		$now = Zend_Date::now();
		$nowSub30 = Zend_Date::now();
		$nowSub30->subMinute(30);
		$select = $scans->select()->where('status = ?', self::READY)
				->where(' CONVERT_TZ(date,
				(SELECT timezone FROM companies WHERE companies.id = scans.company_id),\'SYSTEM\'
				) between \''.$nowSub30->toString(Zend_Date::ISO_8601).'\' and \''.$now->toString(Zend_Date::ISO_8601).'\' ' 
				);
		$scans = $scans->fetchAll($select);
		
		return $scans;
	}
	
	
}
