<?php
/**
 * User handler
 *
 */
class VSCAN_User
{
	const ROLE_KEY = 'role';
	const COMPANY_KEY = 'company';
	/**
	 * 
	 * Create User
	 * @param Array $params
	 * @throws Exception
	 */
	public static function create($params) {
		$db = Zend_Db_Table::getDefaultAdapter();
		$db->beginTransaction();
		try {
			if (!empty($params['id'])) {
				$user = new VSCAN_Model_User();
				$user->find($params['id']);
				if (!empty($params['password'])) {
					$user->setNewPassword($params['password']);
				}
				unset($params['password']);
				$user->populate($params);
			} else {
				$user = new VSCAN_Model_User($params);
			}
			
			$user->save();
			
			//role
			self::updateRoleUser($user->getId(),$params[self::ROLE_KEY]);
			//company
			self::updateCompanyUser($user->getId(), $params[self::COMPANY_KEY]);
			$db->commit();
			return $user;
		} catch (Exception $e) {
			$db->rollBack();
			throw $e;
		}
	}
	/**
	 * 
	 * Enter description here ...
	 * @param unknown_type $userId
	 */
	public static function delete($userId) {
		
		$db = Zend_Db_Table::getDefaultAdapter();
		$db->beginTransaction();
		try {
			self::deleteRelationRole($userId);
			self::deleteRelationCompany($userId);
			$user = new VSCAN_Model_User();
			$user->find($userId);
			$user->delete();
			$db->commit();
			return true;
		} catch (Exception $e) {
			$db->rollBack();
			throw $e;
		}
	}
	
	/**
	 * 
	 * List of companies
	 * @param string|int $userId
	 */
	public static function getCompaniesByUserId($userId)
	{
		$userCompany = new VSCAN_Model_CompanyUser();
		$userCompanies = $userCompany->fetchAll(
			$userCompany->select()
				->where(" user_id = ? ", $userId)
		);
		$companies = array();
		foreach ($userCompanies as $userCompany){
			$companies[] = $userCompany->getCompany();
		}
		return $companies;
	}
	
	/**
	 * 
	 * List of companies names
	 * @param string|int $userId
	 */
	public static function getCompaniesNamesByUserId($userId)
	{
		$companies = self::getCompaniesByUserId($userId);
		$companiesNames = array();
		foreach ($companies as $company) {
	    	$companiesNames[$company->getId()] = $company->getName();
	    }
	    return $companiesNames;
	}
	
	/**
	 * 
	 * List of roles
	 * @param string|int $userId
	 */
	public static function getRolesByUserId($userId)
	{
		$roleUser = new VSCAN_Model_RoleUser();
		$rolesUser = $roleUser->fetchAll(
			$roleUser->select()
				->where(" user_id = ? ", $userId)
		);
		$roles = array();
		foreach ($rolesUser as $roleUser){
			$roles[] = $roleUser->getRole();
		}
		return $roles;
	}
	/**
	 * 
	 * List of roles names
	 * @param string|int $userId
	 */
	public static function getRolesNamesByUserId($userId)
	{
		$roles = self::getRolesByUserId($userId);
		$rolesNames = array();
		foreach ($roles as $role) {
	    	$rolesNames[$role->getId()] = $role->getName();
	    }
	    return $rolesNames;
	}
	
	public static function hasCompany(VSCAN_Model_User $user, $companyId) {
		
		$acl = new VSCAN_Acl($user);
		if ($acl->isUserAllowed('allCompanies')) {
			//user should be an admin
			return true;	
			
		} else {
			$companyUser = new VSCAN_Model_CompanyUser();
			$companyUser->fetchRow(
				$companyUser->select()
					->where('user_id = ?', $user->id)
					->where('company_id = ?', $companyId)
			);
			return $companyUser->isMapped();
		}
	}
	
	/**
	 * 
	 * Update de relation
	 * @param int $userId
	 * @param int|VSCAN_Model_Company $company
	 */
	private static function updateCompanyUser($userId, $company) {
		//company
		$companyUser = new VSCAN_Model_CompanyUser();
		$companyUser->fetchRow(
			$companyUser->select()->where('user_id = ?',$userId)
		);

		$companyUser->setCompany($company);
		$companyUser->setUser($userId);
		$companyUser->save();
	}
	
	/**
	 * 
	 * Update de relation
	 * @param int $userId
	 * @param int|VSCAN_Model_Role $role
	 */
	private static function updateRoleUser($userId, $role) {
		
		$roleUser = new VSCAN_Model_RoleUser();
		$roleUser->fetchRow(
			$roleUser->select()->where('user_id = ?',$userId)
		);

		$roleUser->setRole($role);
		$roleUser->setUser($userId);
		$roleUser->save();
	}
	/**
	 * 
	 * delete de all de companies related with an User
	 * @param int $userId
	 */
	private static function deleteRelationCompany($userId){
		$companyUsers = new VSCAN_Model_CompanyUser();
		$companyUsers = $companyUsers->fetchAll(
			$companyUsers->select()->where('user_id = ?',$userId)
		);
		foreach ($companyUsers as $companyUser) {
			$companyUser->delete();
		}
	}
	/**
	 * 
	 * delete the relation roles related with an User
	 * @param int $userId
	 */
	private static function deleteRelationRole($userId){
		$roleUsers = new VSCAN_Model_RoleUser();
		$roleUsers = $roleUsers->fetchAll(
			$roleUsers->select()->where('user_id = ?',$userId)
		);
		foreach ($roleUsers as $roleUser) {
			$roleUser->delete();	
		}
	}
}


